public class DotMakerHashtable {

  private final HashMap hashMap;
  private final StringBuilder b = new StringBuilder();
  private int nextId = 0;
  private float width;
  
  public static String makeDot(HashMap hashMap) {
    return new DotMakerHashtable(hashMap).getDot();
  }

  public static String makeDot(HashMap hashMap, float width) {
    return new DotMakerHashtable(hashMap, width).getDot();
  }
  
  private DotMakerHashtable(HashMap hashMap) {
    this(hashMap, 2.9f);
  }

  private DotMakerHashtable(HashMap hashMap, float width) {
    this.hashMap = hashMap;
    this.width = width;
  }

  private String getDot() {
    b.append("digraph MyGraph {\n");
    b.append("nodesep=.05;\nrankdir=LR;\n");
    table();
    b.append("}");
    return b.toString();
  }

  private void table() {
    b.append("node [shape=record,width=.1,height=.1]\n\n");
    b.append(String.format("tab[width=%.1f,", width));
    b.append("label=\"");
    for (HashMap.Entry n: hashMap.tab) {
      if (n != null) {
        if (!n.isDeleted) {
          b.append(escapeSpaces(n.key.toString()));
        } else {
          b.append("--- DELETED ---");
        }
      } else {
        b.append('-');
      }
      b.append('|');
    }
    if(hashMap.tab.length > 0) {
      b.delete(b.length()-1, b.length());
    }
    b.append("\",");
    b.append("]");
    b.append(";\n\n");
  }
  
  private String escapeSpaces(String s) {
    StringBuilder b = new StringBuilder(s);
    for (int i=b.length()-1; i >= 0; i--) {
      if (b.charAt(i) == ' ') {
        b.insert(i, '\\');
      }
    }
    return b.toString();
  }
}